# -*- coding: utf-8 -*-

import fcntl
import os
import shutil
import subprocess

from . import templates
from .config import Config
from .paths import HOSTS_FILE, HOSTS_LOCK, LXC_ROOT, NGINX_DIR

def prepare_container():
    # Extract the variables from values given via lxc.hook.pre-start hook
    app = os.environ['LXC_NAME']
    # Remove ephemeral layer data
    clean_ephemeral_layer(app)
    # Configure host and common params used in the app
    configure_app(app)

def clean_ephemeral_layer(app):
    # Cleans containers ephemeral layer.
    # This is done early in the container start process, so the inode of the delta0 directory must remain unchanged
    layer = os.path.join(LXC_ROOT, app, 'delta0')
    if os.path.exists(layer):
        for item in os.scandir(layer):
            shutil.rmtree(item.path) if item.is_dir() else os.unlink(item.path)

def register_container():
    # Extract the variables from values given via lxc.hook.start-host hook
    app = os.environ['LXC_NAME']
    pid = os.environ['LXC_PID']
    # Lease the first unused IP to the container
    ip = update_hosts_lease(app, True)
    # Set IP in container based on PID given via lxc.hook.start-host hook
    cmd = 'ip addr add {}/16 broadcast 172.17.255.255 dev eth0 && ip route add default via 172.17.0.1'.format(ip)
    subprocess.run(['nsenter', '-a', '-t', pid, '--', '/bin/sh', '-c', cmd])

def unregister_container():
    # Extract the variables from values given via lxc.hook.post-stop hook
    app = os.environ['LXC_NAME']
    # Release the container IP
    update_hosts_lease(app, False)
    # Remove ephemeral layer data
    clean_ephemeral_layer(app)

def update_hosts_lease(app, is_request):
    # This is a poor man's DHCP server which uses /etc/hosts as lease database
    # Leases the first unused IP from range 172.17.0.0/16
    # Uses file lock as interprocess mutex
    ip = None
    with open(HOSTS_LOCK, 'w') as lock:
        fcntl.lockf(lock, fcntl.LOCK_EX)
        # Load all existing records
        with open(HOSTS_FILE, 'r') as f:
            leases = [l.strip().split(' ', 1) for l in f]
        # If this call is a request for lease, find the first unassigned IP
        if is_request:
            used_ips = [l[0] for l in leases]
            for i in range(2, 65278): # Reserve last /24 subnet for VPN
                ip = '172.17.{}.{}'. format(i // 256, i % 256)
                if ip not in used_ips:
                    leases.append([ip, app])
                    break
        # Otherwise it is a release in which case we just delete the record
        else:
            leases = [l for l in leases if l[1] != app]
        # Write the contents back to the file
        with open(HOSTS_FILE, 'w') as f:
            for lease in leases:
                f.write('{} {}\n'.format(lease[0], lease[1]))
    return ip

def configure_app(app):
    # Supply common configuration for the application. Done as part of container preparation during service startup
    script = os.path.join('/srv', app, 'update-conf.sh')
    if os.path.exists(script):
        conf = Config()
        setup_env = os.environ.copy()
        setup_env['DOMAIN'] = conf['host']['domain']
        setup_env['PORT'] = conf['host']['port']
        setup_env['EMAIL'] = conf['common']['email']
        setup_env['GMAPS_API_KEY'] = conf['common']['gmaps-api-key']
        subprocess.run([script], env=setup_env, check=True)
